package com.gct.edge.portal.constraint;

import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import com.gct.edge.portal.constant.JobConstant;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.StampedLock;

/**
 * dec: JobActuatorRefrence
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/12/11 下午3:56
 */
@Slf4j
public abstract class JobActuatorReference<T> extends QuartzJobBean {

    private static ConcurrentHashMap<String, StampedLock> lock = new ConcurrentHashMap<>();

    static {
        for (JobConstant value : JobConstant.values()) {
            lock.put(value.name(), new StampedLock());
        }
    }

    private JobDataMap dataMap;

    private String getJobName() {
        String o = (String) dataMap.get(JobConstant.JOB_NAME.name());
        if (StringUtil.isBlankOrEmpty(o)) return "";
        return o;
    }

    private String getJobGroup() {
        String o = (String) dataMap.get(JobConstant.JOB_GROUP.name());
        if (StringUtil.isBlankOrEmpty(o)) return "";
        return o;
    }

    private long getSerialLock() {
        try {
            return lock.get(getJobName()).tryWriteLock(100, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            return 0l;
        }
    }
    private void releaseSerialLock(long stamp){
        lock.get(getJobName()).unlockWrite(stamp);
    }

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        this.dataMap = context.getJobDetail().getJobDataMap();
        long writeLock = getSerialLock();
        if (writeLock == 0l) {
            log.info("Job --- [{},{}] wait for last task.", getJobGroup(), getJobName());
            return;
        }

        try {
            log.info("Start job ======> [{},{}]", getJobGroup(), getJobName());
            List<T> data = external();
            CountDownLatch count = new CountDownLatch(data.size());
            data.stream().parallel().forEach(x -> {
                try {
                    Result result = parallelInternal(x);
                    if (result.getStatus() == -1)
                        log.error("JOB-INNER--[{},{}] run error,data:{},cause:{}", getJobGroup(), getJobName(), JSONObject.toJSONString(x), JSONObject.toJSONString(result));
                    else
                        log.info("JOB-INNER--[{},{}] run successfully,result:{}", getJobGroup(), getJobName(), result.getData());
                } catch (Exception e) {
                    log.error("JOB-INNER--[{},{}] run error,data:{},cause:{}", getJobGroup(), getJobName(), JSONObject.toJSONString(x), e.getMessage());
                } finally {
                    count.countDown();
                }
            });
            try {
                count.await();
            } catch (InterruptedException e) {

            }
            log.info("*<====== [{},{}] completed", getJobGroup(), getJobName());
        } catch (Exception e) {
            log.error("*<====== [{},{}] completed with errors ,cause:{}", getJobGroup(), getJobName(), e.getMessage());
        } finally {
            releaseSerialLock(writeLock);
        }

    }

    public abstract List<T> external();

    public abstract Result parallelInternal(T x);
}
