package com.gct.edge.portal.feign;

import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestParam;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.time.LocalDateTime;
import java.util.concurrent.ConcurrentHashMap;

/**
 * dec:
 * <p>
 * ActuatorClientWrapper
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2021/11/22 下午1:52
 */
@Component
public class ActuatorClientWrapper {

    // region wrapper implement
    private final ActuatorClient client;

    /**
     * hash load balance reference
     */
    public static ThreadLocal<String> lbKey = new ThreadLocal<>();
    /**
     * 方法级隔离
     * 方法内module key隔离
     * module key级序列化执行
     */
    private final static ConcurrentHashMap<Method, ConcurrentHashMap<String, Integer>> methodSerialControl;

    static {
        methodSerialControl = new ConcurrentHashMap<>();
        for (Method method : ActuatorClientMethodEnum.values()) {
            methodSerialControl.put(method, new ConcurrentHashMap<>());
        }
    }


    public ActuatorClientWrapper(ActuatorClient client) {
        this.client = client;


    }

    private Result methodInterceptor(String moduleKey, Method method, Object... args) {
        if (method == null) throw new RuntimeException("method not found");
        if (StringUtil.isBlankOrEmpty(moduleKey))
            return Result.error("Must set moduleKey before calling the interface => " + method);
        ConcurrentHashMap<String, Integer> serial = methodSerialControl.get(method);
        if (serial.putIfAbsent(moduleKey, moduleKey.hashCode()) != null)
            return Result.error("Blocked by serialization,you can try again later.The interface => " + method);
        lbKey.set(moduleKey);
        try {
//            Method reflectiveFeignMethod = client.getClass().getMethod(method.getName(), method.getParameterTypes());
            return JSONObject.parseObject(method.invoke(this.client, args).toString(), Result.class);
        } catch (Exception e) {
            System.out.println(e);
            if(e instanceof InvocationTargetException)
                return Result.error(((InvocationTargetException) e).getTargetException().getMessage());
            return Result.error(e.getMessage());
        } finally {
            lbKey.remove();
            serial.remove(moduleKey);
        }
    }

    // endregion


    public Result loadLocalFile(String path, String moduleKey, String uploadChoose) {
        return methodInterceptor(moduleKey, ActuatorClientMethodEnum.valueOf(0), path, moduleKey, uploadChoose);
    }

    public Result loadRemoteFile(String moduleKey, String ipAddress, Integer port, String authUser, String authPassword, String remotePath) {
        return methodInterceptor(moduleKey, ActuatorClientMethodEnum.valueOf(1), moduleKey, ipAddress, port, authUser, authPassword, remotePath);
    }

    public Result copyAoidConfigData(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(2), jsonObject);
    }

    public Result getAoidConfigDataList(String moduleKey, String proTime) {
        return methodInterceptor(moduleKey, ActuatorClientMethodEnum.valueOf(3), moduleKey, proTime);
    }

    public Result addAoidConfigData(JSONObject aoidConfigData) {
        return methodInterceptor(aoidConfigData.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(4), aoidConfigData);
    }

    public Result addAoidConfigDataWithList(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(5), jsonObject);
    }

    public Result editAoidConfigData(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(6), jsonObject);
    }

    public Result syncModuleConfig(String moduleKey, String proTime, String ipAddress, Integer port, String authUser, String authPassword, String remotePath, String tableName, String actualFileName) {
        return methodInterceptor(moduleKey, ActuatorClientMethodEnum.valueOf(7), moduleKey, proTime, ipAddress, port, authUser, authPassword, remotePath, tableName, actualFileName);
    }

    public Result TimingSummary(String moduleKey, String wellIdList, String startTime) {

        return methodInterceptor(moduleKey, ActuatorClientMethodEnum.valueOf(8),moduleKey, wellIdList, startTime);

    }

    public Result saveDgnsCardLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(9), jsonObject);
    }

    public Result saveAoidGkModelLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(10), jsonObject);
    }
    public Result editCardLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(11), jsonObject);
    }
    public Result copyCardLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("sourceModule"), ActuatorClientMethodEnum.valueOf(12), jsonObject);
    }

    public Result editAoidGKModelLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(13), jsonObject);
    }
    public Result copyAoidGkModelLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("sourceModule"), ActuatorClientMethodEnum.valueOf(14), jsonObject);
    }
    public Result deleteCardLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(15), jsonObject);
    }
    public Result deleteAoidGkModelLib(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(16), jsonObject);
    }

    public Result getMostNewGtcvrRuntimeConfig(String moduleKey){
        return methodInterceptor(moduleKey, ActuatorClientMethodEnum.valueOf(17), moduleKey);
    }

    public Result SaveGtcvrRuntimeConfig(JSONObject jsonObject){
        return  methodInterceptor(jsonObject.getString("moduleKey"),ActuatorClientMethodEnum.valueOf(18), jsonObject);
    }

    public Result copyGlobelConfig(JSONObject jsonObject){
        return  methodInterceptor(jsonObject.getString("moduleKey"),ActuatorClientMethodEnum.valueOf(19), jsonObject);
    }

    public Result deleteGlobelConfig( String moduleKey,  String proTime){
        return  methodInterceptor(moduleKey,ActuatorClientMethodEnum.valueOf(20), moduleKey,proTime);
    }
    public Result getMostNewSinglewellConfig( String moduleKey){
        return  methodInterceptor(moduleKey,ActuatorClientMethodEnum.valueOf(21), moduleKey);
    }

    public Result addSinglewellConfig(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(22), jsonObject);
    }

    public Result updateSinglewellConfig(JSONObject jsonObject) {
        return methodInterceptor(jsonObject.getString("moduleKey"), ActuatorClientMethodEnum.valueOf(23), jsonObject);
    }
    public Result deleteSinglewellConfig( String moduleKey,  String proTime){
        return  methodInterceptor(moduleKey,ActuatorClientMethodEnum.valueOf(24), moduleKey,proTime);
    }

    public Result copySinglewellConfig(JSONObject jsonObject){
        return  methodInterceptor(jsonObject.getString("moduleKey"),ActuatorClientMethodEnum.valueOf(25), jsonObject);
    }

    public Result addDgnsGtgktype(JSONObject jsonObject){
        return  methodInterceptor(jsonObject.getString("moduleKey"),ActuatorClientMethodEnum.valueOf(26), jsonObject);
    }
    public Result copyDgnsGtgktype(JSONObject jsonObject){
        return  methodInterceptor(jsonObject.getString("moduleKey"),ActuatorClientMethodEnum.valueOf(27), jsonObject);
    }

    public Result deleteDgnsGtgktype( String moduleKey,  String proTime){
        return  methodInterceptor(moduleKey,ActuatorClientMethodEnum.valueOf(28), moduleKey,proTime);
    }

    public Result editSingleWellConfig( JSONObject jsonObject){
        return  methodInterceptor(jsonObject.getString("moduleKey"),ActuatorClientMethodEnum.valueOf(29), jsonObject);
    }

    public Result deleteSingleWellConfig( JSONObject jsonObject){
        return  methodInterceptor(jsonObject.getString("moduleKey"),ActuatorClientMethodEnum.valueOf(30), jsonObject);
    }
}
