package com.gct.edge.portal.job;

import com.gct.common.util.StringUtil;
import com.gct.edge.base.util.DataBaseUtil;
import com.gct.edge.base.util.DataSourceUtil;
import com.gct.edge.portal.entity.EdgeModuleConfigInfo;
import com.gct.edge.portal.entity.EdgeModuleConfigInfoMetric;
import com.gct.edge.portal.entity.EdgeShardDbInfo;
import com.gct.edge.portal.properties.InitModuleSpaceProperties;
import com.gct.edge.portal.service.EdgeModuleConfigInfoService;
import com.gct.edge.portal.service.EdgeShardDbInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.util.ResourceUtils;

import javax.sql.DataSource;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.stream.Collectors;

@Slf4j
public class AutoInitModuleSpaceJob extends QuartzJobBean {
    @Autowired
    EdgeShardDbInfoService dbInfoService;

    @Autowired
    EdgeModuleConfigInfoService moduleConfigInfoService;
    @Autowired
    InitModuleSpaceProperties enableSqlProperties;
    private final static ReadWriteLock lock = new ReentrantReadWriteLock();

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) {
        boolean b = false;
        Lock writeLock = AutoInitModuleSpaceJob.lock.writeLock();
        try {
            b = writeLock.tryLock(10, TimeUnit.MILLISECONDS);
        } catch (InterruptedException ignored) {
        }
        if (!b) return;
        try {
            initModuleSpace();
        } catch (Exception ignore) {
            ignore.printStackTrace();
        }

        writeLock.unlock();
    }

    final private Map<String, DataSource> sourceMap = new HashMap<>();

    private void initModuleSpace() {
        List<EdgeModuleConfigInfo> waiting = moduleConfigInfoService.getListByCreatedCondition(new EdgeModuleConfigInfo()
                .setIsLoaded(1)
                .setLastTableCreatedTime(LocalDateTime.now().plusMonths(1)));
        if (Objects.isNull(waiting) || waiting.isEmpty()) return;

        Map<String, EdgeShardDbInfo> actualWeight = dbInfoService.getAllMaps();

        if (actualWeight.isEmpty()) {
            log.warn("Not available data source to allocate edge database");
            return;
        }

        List<EdgeModuleConfigInfoMetric> metrics = moduleConfigInfoService.getCountGroupByDBKey(new EdgeModuleConfigInfo().setIsLoaded(2));

        Map<String, EdgeModuleConfigInfoMetric> metricMap = metrics.stream().collect(Collectors.toMap(EdgeModuleConfigInfoMetric::getDbKey, metric -> metric, (a, b) -> b));

        for (EdgeModuleConfigInfo created : waiting) {
            //加入动态增减、重新负载逻辑
            EdgeShardDbInfo expectedDB =
                    created.getIsLoaded() == 1 || StringUtil.isBlankOrEmpty(created.getDbKey()) || Objects.isNull(actualWeight.get(created.getDbKey())) ?
                            chooseExpectedDB(actualWeight, metricMap) : actualWeight.get(created.getDbKey());

            if (initDB(expectedDB, created)) {
                moduleConfigInfoService.updateLoadByKey(new EdgeModuleConfigInfo()
                        .setModuleKey(created.getModuleKey())
                        .setIsLoaded(2)//completed
                        .setUpdateTime(LocalDateTime.now())
                        .setDbId(expectedDB.getId())
                        .setDbKey(expectedDB.getDbKey())
                        .setId(created.getId())
                        .setLastTableCreatedTime(LocalDateTime.now().plusMonths(generateMons)));
                EdgeModuleConfigInfoMetric end = metricMap.get(expectedDB.getDbKey());
                if (end == null) {
                    end = new EdgeModuleConfigInfoMetric().setCount(0l)
                            .setDbKey(expectedDB.getDbKey())
                            .setDbId(expectedDB.getId());
                }
                end.setCount(end.getCount() + 1);
                metricMap.put(expectedDB.getDbKey(), end);
            }
        }
    }

    private boolean initDB(EdgeShardDbInfo expectedDB, EdgeModuleConfigInfo created) {
        DataSource dataSource = sourceMap.get(expectedDB.getDbKey());
        if (dataSource == null) {
            dataSource = DataSourceUtil.createSingleDataSource(expectedDB.getJdbcProperty());
            sourceMap.put(expectedDB.getDbKey(), dataSource);
        }

        DataBaseUtil util = new DataBaseUtil(dataSource);
        boolean b = util.constructSchemaByModuleKey(created.getModuleKey());
        if (!b) {
            log.error("Created failed edge schema for module key:{}", created.getModuleKey());
            return false;
        }

        Collection<String> postfix = generateTablePostfix(created.getLastTableCreatedTime());
        if (postfix.isEmpty()) {
            log.warn("Generated logic table name error,module key:{},to check the value of 'LastTableCreatedTime':{}", created.getModuleKey(), created.getLastTableCreatedTime());
            return true;
        }
        for (String s : postfix) {
            if (!enableSqlProperties.sql.isEmpty()) {
                for (String sqlName : enableSqlProperties.sql) {
                    String sql = getFileContentFromClasspath("sql/" + sqlName + ".sql");
                    b = util.createTableByFix(created.getModuleKey(), s, sql);
                    if (!b) break;
                }
            }
        }
        if (!enableSqlProperties.onlyShardDb.isEmpty()) {
            for (String sqlName : enableSqlProperties.onlyShardDb) {
                String sql = getFileContentFromClasspath("sql/" + sqlName + ".sql");
                b = util.createTableByFix(created.getModuleKey(),"", sql);
                if (!b) break;
            }
        }
        if (!b) log.error("Created failed edge table  for module key:{}", created.getModuleKey());

        return b;
    }

    private String getFileContentFromClasspath(String path) {
        try {
            StringBuilder sb = new StringBuilder();
            InputStream resourceAsStream = getClass().getClassLoader().getResourceAsStream(path);
            BufferedReader br = new BufferedReader(new InputStreamReader(resourceAsStream, "UTF-8"));
            String line;
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            return sb.toString();
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    private static int generateMons = 3;

    /**
     * 生成最新三个月的 postfix of table name
     *
     * @param start 参数：开始时间
     * @return Collection
     */
    private Collection<String> generateTablePostfix(LocalDateTime start) {

        Set<String> re = new HashSet<>();
        if (Objects.isNull(start)) return re;
        LocalDateTime end = LocalDateTime.now().plusMonths(generateMons);
        if (start.isAfter(end)) return re;
        DateTimeFormatter postfixFormat = DateTimeFormatter.ofPattern("yyyyMM");
        re.add(start.format(postfixFormat));
        for (; start.isBefore(end); start = start.plusMonths(1)) {
            re.add(start.format(postfixFormat));
        }
        re.add(end.format(postfixFormat));
        return re;
    }

    private EdgeShardDbInfo chooseExpectedDB(Map<String, EdgeShardDbInfo> actualWeight, Map<String, EdgeModuleConfigInfoMetric> metrics) {
        double planWeightTotal = 0;
        for (EdgeShardDbInfo edgeShardDbInfo : actualWeight.values()) {
            planWeightTotal += edgeShardDbInfo.getWeight();
        }

        TreeMap<Double, TreeMap<Double, EdgeShardDbInfo>> re = new TreeMap<>();
        for (Map.Entry<String, EdgeShardDbInfo> item : actualWeight.entrySet()) {
            if (Objects.isNull(metrics.get(item.getKey()))) {
                Double key = item.getValue().getWeight() / planWeightTotal;
                if (re.get(key) == null) {
                    TreeMap<Double, EdgeShardDbInfo> temp = new TreeMap<>();
                    temp.put(Double.valueOf(item.getValue().getWeight().toString()), item.getValue());
                    re.put(key, temp);
                } else re.get(key).put(key, item.getValue());
            }
        }

        if (metrics.isEmpty()) return re.lastEntry().getValue().lastEntry().getValue();

        double metricTotal = 0;
        for (EdgeModuleConfigInfoMetric metric : metrics.values()) {
            if (Objects.nonNull(actualWeight.get(metric.getDbKey())))//不存在的库不再处理
                metricTotal += metric.getCount();
        }
        for (EdgeModuleConfigInfoMetric metric : metrics.values()) {
            if (Objects.isNull(actualWeight.get(metric.getDbKey()))) continue;
            EdgeShardDbInfo ac = actualWeight.get(metric.getDbKey());
            double v = ac.getWeight() / planWeightTotal;
            v = v - (metric.getCount() / metricTotal);
            if (v < 0) continue;//过滤比例超标的db key候选
            double key = ac.getWeight();
            if (re.get(v) == null) {
                TreeMap<Double, EdgeShardDbInfo> temp = new TreeMap<>();
                temp.put(key, ac);
                re.put(v, temp);
            } else re.get(v).put(key, ac);
        }
        return re.lastEntry().getValue().lastEntry().getValue();
    }
}
