package com.gct.edge.portal.job;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.base.entity.TableNameEnum;
import com.gct.edge.portal.entity.EdgeModuleConfigInfo;
import com.gct.edge.portal.entity.EdgeSyncConfigMonitor;
import com.gct.edge.portal.feign.ActuatorClientWrapper;
import com.gct.edge.portal.service.EdgeModuleConfigInfoService;
import com.gct.edge.portal.service.EdgeSyncConfigMonitorService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;

import javax.annotation.Nullable;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.StampedLock;

/**
 * @author xusirui 2021/12/3
 */
@Slf4j
public class ConfigureModuleRemotelyJob extends QuartzJobBean {

    @Autowired
    EdgeSyncConfigMonitorService edgeSyncConfigMonitorService;

    @Autowired
    EdgeModuleConfigInfoService edgeModuleConfigInfoService;

    @Autowired
    ActuatorClientWrapper actuatorClient;

    private static final StampedLock lock = new StampedLock();

    @Override
    protected void executeInternal(@Nullable JobExecutionContext context) throws JobExecutionException {
        long writeLock;
        try {
            writeLock = lock.tryWriteLock(100, TimeUnit.MILLISECONDS);
            if (writeLock == 0L) return;
        } catch (InterruptedException e) {
            return;
        }

        List<EdgeSyncConfigMonitor> list = edgeSyncConfigMonitorService.list(
                new QueryWrapper<EdgeSyncConfigMonitor>()
                        .eq("release_status", 1)
        );
        CountDownLatch count = new CountDownLatch(list.size());
        list.stream().parallel().forEach(x -> {
            try {
                execSync(x);
            } catch (Exception e) {
                log.error("Run sync module data failed. module:(id={},moduleKey={},cause:{}", x.getId(), x.getModuleKey(), e.getMessage());
            } finally {
                edgeSyncConfigMonitorService.updateById(x);
                count.countDown();
            }
        });
        try {
            count.await();
        } catch (InterruptedException e) {
            throw new JobExecutionException(e);
        }
        lock.unlockWrite(writeLock);
    }

    public void execSync(EdgeSyncConfigMonitor x) {
        EdgeModuleConfigInfo one = edgeModuleConfigInfoService.getOne(
                new QueryWrapper<EdgeModuleConfigInfo>()
                        .eq("module_key", x.getModuleKey())
        );
        if (Objects.isNull(one)) {
            log.error("sync config failed, {} info doesn't exist", x.getModuleKey());
            return;
        }
        TableNameEnum tableNameEnum = TableNameEnum.valueOf(x.getContentType());
        if (tableNameEnum == null) {
            log.error("sync config failed, contentType {} need a map table", x.getContentType());
            return;
        }
        try {
            Result r = actuatorClient.syncModuleConfig(x.getModuleKey(), DateTimeUtil.formatString(x.getReleaseVersion()), one.getIpAddress(), one.getPort(), one.getAuthUser(),
                    one.getAuthPassword(), one.getRemotePath()
                    , tableNameEnum.getTableName(), tableNameEnum.getActualFileName());
            if (r.getStatus() == 1) {
                x.setReleaseTime(LocalDateTime.now());
                x.setReleaseStatus(2);
            }
            System.out.println(r);
        } catch (Exception e) {
            log.error("sync config failed, call actuator module syncModuleConfig method failed, cause by {}", e.getMessage());
        }

    }
}
