package com.gct.edge.portal.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.CodeMsg;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.portal.entity.EdgeGroupInfo;
import com.gct.edge.portal.mapper.EdgeGroupInfoMapper;
import com.gct.edge.portal.service.EdgeGroupInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lloyd
 * @since 2021-10-26
 */
@Service
public class EdgeGroupInfoServiceImpl extends ServiceImpl<EdgeGroupInfoMapper, EdgeGroupInfo> implements EdgeGroupInfoService {

    @Autowired
    private EdgeGroupInfoService edgeGroupInfoService;

    @Autowired
    private EdgeGroupInfoMapper edgeGroupInfoMapper;

    /**
     * 添加分组信息
     *
     * @param edgeGroupInfo
     * @return
     */
    @Override
    @Transactional
    public Result addEdgeGroupInfo(EdgeGroupInfo edgeGroupInfo) {
        Integer parentId = edgeGroupInfo.getParentId();
        Integer sort = edgeGroupInfoMapper.getMaxSort() + 1;
        if (this.findGroupByName(edgeGroupInfo.getGroupName()) != null) { //分组名验证
            return Result.create(CodeMsg.createCodeMsg(-1, "该分组已经存在"));
        }
        edgeGroupInfo.setSort(sort);
        edgeGroupInfo.setCreateTime(LocalDateTime.now());

        if (parentId != 0) {
            EdgeGroupInfo parent = this.getById(parentId);
            edgeGroupInfo.setLevel(parent.getLevel() + 1);
            edgeGroupInfo.setOrderField(parent.getOrderField() + "/" + sort);
        } else {
            edgeGroupInfo.setLevel(0);
            edgeGroupInfo.setOrderField(String.valueOf(sort));
        }
        int insert = edgeGroupInfoMapper.insert(edgeGroupInfo);
        return insert == 1 ? Result.create(CodeMsgEnum.ADD_SUCCESS) : Result.create(CodeMsgEnum.ADD_FAIL);
    }


    /**
     * 分组名称验证
     *
     * @param groupName
     * @return
     */
    public EdgeGroupInfo findGroupByName(String groupName) {
        QueryWrapper<EdgeGroupInfo> queryWrapper = new QueryWrapper<>();   //创建查询queryWrapper
        queryWrapper.eq("group_name", groupName);
        return edgeGroupInfoMapper.selectOne(queryWrapper);
    }

    /**
     * 修改分组信息
     *
     * @param edgeGroupInfo
     * @return
     */
    public Result updateEdgeGroupInfo(EdgeGroupInfo edgeGroupInfo) {
        LocalDateTime dateTime = LocalDateTime.now();
        edgeGroupInfo.setUpdateTime(dateTime);
        int update = edgeGroupInfoMapper.updateById(edgeGroupInfo);
        return update == 1 ? Result.create(CodeMsgEnum.MODIFY_SUCCESS) : Result.create(CodeMsgEnum.MODIFY_FAIL);
    }


    /**
     * 根据参数条件查询分组属性数据
     *
     * @param pageNum
     * @param pageSize
     * @param group_name
     * @return
     */
    public Result findAllEdgeGroupInfo(Integer pageNum, Integer pageSize, String group_name) {

        Page<EdgeGroupInfo> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        paramPage.addOrder(OrderItem.desc("update_time"));

        IPage<EdgeGroupInfo> page = baseMapper.findAllEdgeGroupInfo
                (paramPage, group_name);
        return Result.ok(page);

    }


    @Override
    public Result findTree() {
        JSONArray rootArr = new JSONArray();
        Map<Integer, JSONObject> tree = new HashMap<>();
        List<EdgeGroupInfo> orgList = baseMapper.selectList(new QueryWrapper<EdgeGroupInfo>().last("order by id"));
        orgList.forEach(x -> {
            JSONObject obj = (JSONObject) JSON.toJSON(x);
            obj.put("createTime", DateTimeUtil.formatString(x.getCreateTime()));
            obj.put("updateTime", DateTimeUtil.formatString(x.getUpdateTime()));
            if (x.getParentId() == null || x.getParentId() == 0) rootArr.add(obj);
            tree.put(x.getId(), obj);
        });
        orgList.forEach(x -> {
            JSONObject parent = tree.get(x.getParentId());
            if (parent != null) {
                JSONArray children = parent.getJSONArray("children");
                if (children == null) {
                    children = new JSONArray();
                    parent.put("children", children);
                }
                children.add(tree.get(x.getId()));
            }
        });
        return Result.ok(rootArr);
    }

    @Override
    public Result findList() {
        QueryWrapper<EdgeGroupInfo> wrapper = new QueryWrapper<>();
        wrapper.orderByAsc("order_field");
        List<EdgeGroupInfo> list = baseMapper.selectList(wrapper);
        return Result.ok(list);
    }

    /**
     * 添加子节点
     *
     * @param groupName
     * @param remark
     * @param parentId
     * @return
     */
    @Override
    public Result addSonEdgeGroupInfo(String groupName, String remark, Integer parentId) {
        Integer sort = edgeGroupInfoMapper.getMaxSort() + 1;
        EdgeGroupInfo edgeGroupInfo = new EdgeGroupInfo();
        if (this.findGroupByName(groupName) != null) { //分组名验证
            return Result.create(CodeMsg.createCodeMsg(-1, "该分组已经存在"));
        }
        edgeGroupInfo.setGroupName(groupName);
        edgeGroupInfo.setParentId(parentId);
        edgeGroupInfo.setRemark(remark);
        edgeGroupInfo.setCreateTime(LocalDateTime.now());
        edgeGroupInfo.setSort(sort);
        if (parentId != 0) {
            EdgeGroupInfo parent = this.getById(parentId);
            edgeGroupInfo.setLevel(parent.getLevel() + 1);
            edgeGroupInfo.setOrderField(parent.getOrderField() + "/" + sort);
        } else {
            edgeGroupInfo.setLevel(0);
            edgeGroupInfo.setOrderField(String.valueOf(sort));
        }
        int insert = edgeGroupInfoMapper.insert(edgeGroupInfo);
        return insert == 1 ? Result.create(CodeMsgEnum.ADD_SUCCESS) : Result.create(CodeMsgEnum.ADD_FAIL);
    }

}
